#region Copyright Syncfusion Inc. 2001 - 2005
//
//  Copyright Syncfusion Inc. 2001 - 2005. All rights reserved.
//
//  Use of this code is subject to the terms of our license.
//  A copy of the current license can be obtained at any time by e-mailing
//  licensing@syncfusion.com. Re-distribution in any form is strictly
//  prohibited. Any infringement will be prosecuted under applicable laws. 
//
#endregion

using System;
using System.Drawing;
using System.Diagnostics;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Text;

using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Grouping;
using Syncfusion.Windows.Forms.Grid.Grouping;

namespace GroupingPerf
{
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class Form1 : System.Windows.Forms.Form {
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		private System.Windows.Forms.Panel gridPanel;
		private GridGroupingControl grid;		

		//		private System.Windows.Forms.LinkLabel lblShow;
		private DataTable table;

		int randomKey = 25;
		int startId = 100;
		int recordCount = 100;
		int endId = 1000;
		bool calculateMaxColumnWidth = false;
		bool useGroupingSortList = false;
		bool sortandCategorize = true;
		bool showAll = true;
		private System.Windows.Forms.LinkLabel lblShow;

		string[] states = new string[]{"North Carolina", "South Carolina", "Washington", "Nevada", "Ohio"};

		public Form1() {
			// GroupingEngineFactory provides a trimmed down GridGroup
			// which eliminates overhead of not needed preview rows, header and footer cells.
			GridEngineFactory.Factory = new GroupingEngineFactory();

			InitializeComponent();

			InitializeDataTable();

			InitializeGroupingGrid();

		}

		private void InitializeDataTable() {
			Random random = new Random(randomKey++);
			int numStates = states.GetLength(0);
			string nameFormat = "State Name{0:00000}";

			this.table = new DataTable("Test");

			DataColumn col = table.Columns.Add();
			col.DataType = typeof(string);
			col.ColumnName = "Name";

			col = table.Columns.Add();
			col.DataType = typeof(string);
			col.ColumnName = "State";

			col = table.Columns.Add();
			col.DataType = typeof(int);
			col.ColumnName = "id";

			for(int i = 0; i < recordCount; i++) {
				DataRow row = this.table.NewRow();
				
				row["State"] = states[random.Next(numStates-1)];

				// To gave negative id for Nevada 
				
				if (row["State"].ToString() == "Nevada") {
					row["id"] = random.Next(endId * -1,startId * -1);
					row["Name"] = "None";
				}
				else {
					row["id"] = random.Next(startId,endId);
					row["Name"] = string.Format(nameFormat, i);
				}
				if (showAll == false) {
					if (Convert.ToInt32(row["id"].ToString()) > 0) {
						this.table.Rows.Add(row);
					}
				}
				else {
					this.table.Rows.Add(row);
				}
			}
			this.table.DefaultView.RowStateFilter = DataViewRowState.CurrentRows;

			//			if (showAll == false)
			//			{
			//				this.table.DefaultView.RowFilter = "id>0";
			//			}
		}

		private void InitializeGroupingGrid() {
			if (this.grid != null)
				this.grid.Dispose();


			// Fill DataTable
			InitializeDataTable();


			// Show an empty grid - triggers JIT compilation when run first time
			this.grid = new GridGroupingControl();
			this.grid.TableDescriptor.AllowCalculateMaxColumnWidth = calculateMaxColumnWidth;
			this.grid.Dock = DockStyle.Fill;
			
			//			this.lblShow = new LinkLabel();
			//			lblShow.Text = "Show All";
			
			//			this.gridPanel.Controls.Add(this.lblShow);

            
			// DataView has performance issues when inserting new records into a large
			// dataset. Instead of getting the DataTable.DefaultView, the engine
			// can wrap the data table with a more efficient DataTableList class. 
			// This allows instantenoues inserts for DataTable with > 100000 records.
			this.grid.Engine.AllowSwapDataViewWithDataTableList = true;

			this.grid.SourceListListChanged += new TableListChangedEventHandler(grid_SourceListListChanged);
			this.grid.TableControl.CellDoubleClick +=new GridCellClickEventHandler(TableControl_CellDoubleClick);
			
			//			this.lblShow.Click += new EventHandler(lblShow_Click);
			// Assign data source
			if (useGroupingSortList) {
				GroupingSortList gs = new GroupingSortList(this.table);
				this.grid.DataSource = gs;
			}
			else
				this.grid.DataSource = this.table;

			// Assign grouping and sort order
			if (this.sortandCategorize) {							
				this.grid.TableDescriptor.GroupedColumns.Add("Id");
				this.grid.TableDescriptor.GroupedColumns.Add("Name");
				this.grid.TableDescriptor.GroupedColumns.Add("State");
				
				//				this.grid.TableDescriptor.GroupedColumns.Add("id");
			}

			this.grid.TableDescriptor.Columns["State"].ReadOnly = true;
			this.grid.TableDescriptor.Columns["State"].Appearance.AnyRecordFieldCell.Enabled = false;
							

			// Force filling and intialization of engine and drawing of grid.
			//			int count = this.grid.Table.DisplayElements.Count;
			//

			// paint the grid.
			
			this.gridPanel.Controls.Add(this.grid);
			this.grid.Update();
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing ) {
			if( disposing ) {
				if (components != null) {
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent() {
			this.gridPanel = new System.Windows.Forms.Panel();
			this.lblShow = new System.Windows.Forms.LinkLabel();
			this.SuspendLayout();
			// 
			// gridPanel
			// 
			this.gridPanel.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.gridPanel.Location = new System.Drawing.Point(80, 72);
			this.gridPanel.Name = "gridPanel";
			this.gridPanel.Size = new System.Drawing.Size(456, 288);
			this.gridPanel.TabIndex = 0;
			// 
			// lblShow
			// 
			this.lblShow.Location = new System.Drawing.Point(88, 48);
			this.lblShow.Name = "lblShow";
			this.lblShow.Size = new System.Drawing.Size(88, 23);
			this.lblShow.TabIndex = 1;
			this.lblShow.TabStop = true;
			this.lblShow.Text = "Show All";
			this.lblShow.LinkClicked += new System.Windows.Forms.LinkLabelLinkClickedEventHandler(this.lblShow_LinkClicked);
			// 
			// Form1
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(656, 566);
			this.Controls.Add(this.lblShow);
			this.Controls.Add(this.gridPanel);
			this.MinimumSize = new System.Drawing.Size(400, 600);
			this.Name = "Form1";
			this.Text = "Grouping Performance";
			this.ResumeLayout(false);

		}
		#endregion


		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() {
			Application.Run(new Form1());
		}


		private void grid_SourceListListChanged(object sender, TableListChangedEventArgs e) {
			// use optimization. Grid is not filtered by Name and we also don't use custom counters.
			// So, as long as the sort order is not modified, there is no need to to invalidate
			// counters bottom up
			e.ShouldInvalidateSummaries = false;
			e.ShouldInvalidateCounters = false;
		}

		//		private void lblShow_Click(object sender, EventArgs e)
		//		{
		//			
		//		}

		private void lblShow_LinkClicked(object sender, System.Windows.Forms.LinkLabelLinkClickedEventArgs e) {
			if (this.lblShow.Text == "Show All") {
				showAll = false;
				InitializeDataTable();
				this.grid.DataSource = this.table;
				this.grid.Update();
				this.lblShow.Text = "Hide Negative";							
			}
			else {
				this.lblShow.Text = "Show All";
				showAll = true;
				InitializeDataTable();
				this.grid.DataSource = this.table;
				this.grid.Update();	
			}
		}
		
		private void TableControl_CellDoubleClick(object sender, GridCellClickEventArgs e) {
			MessageBox.Show("Entered");
		}
	}

	/// <summary>
	/// GroupingSortList wraps a DataView with a IBindingList and implements the IGroupingList
	/// interface and its Sort method. This allows performing the sort on the dataview directly
	/// instead of letting the grouping engine perform the sorting.
	/// </summary>
	public class GroupingSortList : IGroupingList, IBindingList
	{
		DataTable dt;
		DataView dv;
		IBindingList bindingList;

		public GroupingSortList(DataTable dt)
		{
			this.dt = dt;
			this.dv = dt.DefaultView;
			bindingList = (IBindingList) dv;
			bindingList.ListChanged += new ListChangedEventHandler(bindingList_ListChanged);
		}

		#region IGroupingList Members

		public void ApplySort(RelationChildColumnDescriptorCollection relationChildColumns, SortColumnDescriptorCollection groupColumns, SortColumnDescriptorCollection sortColumns)
		{
			bool first = true;
			StringBuilder sb = new StringBuilder();
			foreach (SortColumnDescriptor sd in relationChildColumns)
			{
				if (first)
					first = false;
				else
					sb.Append(", ");

				sb.Append(sd.FieldDescriptor.MappingName);
				if (sd.SortDirection == ListSortDirection.Descending)
					sb.Append(" DESC");
			}
			foreach (SortColumnDescriptor sd in groupColumns)
			{
				if (first)
					first = false;
				else
					sb.Append(", ");

				sb.Append(sd.FieldDescriptor.MappingName);
				if (sd.SortDirection == ListSortDirection.Descending)
					sb.Append(" DESC");
			}
			foreach (SortColumnDescriptor sd in sortColumns)
			{
				if (first)
					first = false;
				else
					sb.Append(", ");

				sb.Append(sd.FieldDescriptor.MappingName);
				if (sd.SortDirection == ListSortDirection.Descending)
					sb.Append(" DESC");
			}

			string sort = sb.ToString();
			if (sort != dv.Sort)
			{
				dv.Sort = sort;
			}
		}

		public bool AllowItemReference
		{
			get
			{
				return false;
			}
		}

		public bool SupportsGroupSorting
		{
			get
			{
				return true;
			}
		}

		public Syncfusion.Grouping.GroupingSortBehavior GroupingSortBehavior
		{
			get
			{
				return Syncfusion.Grouping.GroupingSortBehavior.GroupByGroup;
			}
		}

		#endregion

		#region IBindingList Members

		public void AddIndex(PropertyDescriptor property)
		{
			bindingList.AddIndex(property);
		}

		public bool AllowNew
		{
			get
			{
				return bindingList.AllowNew;
			}
		}

		void System.ComponentModel.IBindingList.ApplySort(PropertyDescriptor property, System.ComponentModel.ListSortDirection direction)
		{
			bindingList.ApplySort(property, direction);
		}

		public PropertyDescriptor SortProperty
		{
			get
			{
				return bindingList.SortProperty;
			}
		}

		public int Find(PropertyDescriptor property, object key)
		{
			return bindingList.Find(property, key);
		}

		public bool SupportsSorting
		{
			get
			{
				return bindingList.SupportsSorting;
			}
		}

		public bool IsSorted
		{
			get
			{
				return bindingList.IsSorted;
			}
		}

		public bool AllowRemove
		{
			get
			{
				return bindingList.AllowRemove;
			}
		}

		public bool SupportsSearching
		{
			get
			{
				return bindingList.SupportsSearching;
			}
		}

		public System.ComponentModel.ListSortDirection SortDirection
		{
			get
			{
				return bindingList.SortDirection;
			}
		}

		public event System.ComponentModel.ListChangedEventHandler ListChanged;

		public bool SupportsChangeNotification
		{
			get
			{
				return bindingList.SupportsChangeNotification;
			}
		}

		public void RemoveSort()
		{
			bindingList.RemoveSort();
		}

		public object AddNew()
		{
			return bindingList.AddNew();
		}

		public bool AllowEdit
		{
			get
			{
				return bindingList.AllowEdit;
			}
		}

		public void RemoveIndex(PropertyDescriptor property)
		{
			bindingList.RemoveIndex(property);
		}

		#endregion

		#region IList Members

		public bool IsReadOnly
		{
			get
			{
				return bindingList.IsReadOnly;
			}
		}

		public object this[int index]
		{
			get
			{
				return bindingList[index];
			}
			set
			{
				bindingList[index] = value;
			}
		}

		public void RemoveAt(int index)
		{
			bindingList.RemoveAt(index);
		}

		public void Insert(int index, object value)
		{
			bindingList.Insert(index, value);
		}

		public void Remove(object value)
		{
			bindingList.Remove(value);
		}

		public bool Contains(object value)
		{

			return bindingList.Contains(value);
		}

		public void Clear()
		{
			bindingList.Clear();
		}

		public int IndexOf(object value)
		{
			return bindingList.IndexOf(value);
		}

		public int Add(object value)
		{
			return bindingList.Add(value);
		}

		public bool IsFixedSize
		{
			get
			{
				return bindingList.IsFixedSize;
			}
		}

		#endregion

		#region ICollection Members

		public bool IsSynchronized
		{
			get
			{
				return bindingList.IsSynchronized;
			}
		}

		public int Count
		{
			get
			{
				return bindingList.Count;
			}
		}

		public void CopyTo(Array array, int index)
		{
			bindingList.CopyTo(array, index);
		}

		public object SyncRoot
		{
			get
			{
				return bindingList.SyncRoot;
			}
		}

		#endregion

		#region IEnumerable Members

		public IEnumerator GetEnumerator()
		{
			return bindingList.GetEnumerator();
		}

		#endregion

		private void bindingList_ListChanged(object sender, ListChangedEventArgs e)
		{
			if (ListChanged != null)
				ListChanged(this, e);
		}
	}

	/// <summary>
	/// GroupingEngineFactory provides a trimmed down GridGroup
	/// which eliminates overhead of not needed preview rows, header and footer cells.
	/// </summary>
	public class GroupingEngineFactory : GridEngineFactoryBase
	{
		public override GridEngine CreateEngine()
		{
			return new GroupingEngine();
		}
	}

	public class GroupingEngine : GridEngine
	{
		public override Group CreateGroup(Section parent)
		{
			return new GroupingGroup(parent);
		}

	}

	public class GroupingGroup : Group
	{
		public GroupingGroup(Section parent)
			: base(parent)
		{
		}

		protected override void OnInitializeSections(bool hasRecords, SortColumnDescriptorCollection fields)
		{
			this.Sections.Add(this.ParentTableDescriptor.CreateCaptionSection(this));
			if (hasRecords)
				this.Sections.Add(this.ParentTableDescriptor.CreateRecordsDetails(this, fields));
			else
				this.Sections.Add(this.ParentTableDescriptor.CreateGroupsDetails(this, fields));
			this.Sections.Add(this.ParentTableDescriptor.CreateSummarySection(this));
		}

		public override bool IsChildVisible(Element el)
		{
			if (el is CaptionSection)
			{
				return true;
			}
			else if (el is DetailsSection)
			{
				return this.IsExpanded;
			}
			else if (el is GridSummarySection)
			{
				return this.IsExpanded;
			}
			return true;
		}
	}

}
